@@ -254,12 +254,6 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
   orc_compiler_rewrite_vars (compiler);
   if (compiler->error) goto error;
 
-  if (compiler->target) {
-    orc_compiler_global_reg_alloc (compiler);
-
-    orc_compiler_rewrite_vars2 (compiler);
-  }
-
 #if 0
   {
     ORC_ERROR("variables");
@@ -285,9 +279,6 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
     }
   }
 #endif
-
-  if (compiler->error) goto error;
-
   program->orccode = orc_code_new ();
 
   program->orccode->is_2d = program->is_2d;
@@ -323,6 +314,13 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
     goto error;
   }
 
+  if (compiler->target) {
+    orc_compiler_global_reg_alloc (compiler);
+
+    orc_compiler_rewrite_vars2 (compiler);
+  }
+  if (compiler->error) goto error;
+
   orc_compiler_assign_rules (compiler);
   if (compiler->error) goto error;
 
@@ -582,6 +580,10 @@ orc_compiler_rewrite_insns (OrcCompiler *compiler)
           cinsn->opcode = get_loadp_opcode_for_size (opcode->src_size[i]);
           cinsn->dest_args[0] = orc_compiler_new_temporary (compiler,
               opcode->src_size[i] * multiplier);
+          if (var->vartype == ORC_VAR_TYPE_CONST) {
+            compiler->vars[cinsn->dest_args[0]].flags |=
+                ORC_VAR_FLAG_VOLATILE_WORKAROUND;
+          }
           cinsn->src_args[0] = insn.src_args[i];
           insn.src_args[i] = cinsn->dest_args[0];
 
@@ -726,7 +728,7 @@ orc_compiler_rewrite_vars (OrcCompiler *compiler)
 
       if (!compiler->vars[var].used) {
         if (compiler->vars[var].vartype == ORC_VAR_TYPE_TEMP) {
-          ORC_COMPILER_ERROR(compiler, "using uninitialized temp var");
+          ORC_COMPILER_ERROR(compiler, "using uninitialized temp var at line %d", insn->line);
           compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
         }
         compiler->vars[var].used = TRUE;
@@ -744,25 +746,25 @@ orc_compiler_rewrite_vars (OrcCompiler *compiler)
         continue;
       }
       if (compiler->vars[var].vartype == ORC_VAR_TYPE_SRC) {
-        ORC_COMPILER_ERROR(compiler,"using src var as dest");
+        ORC_COMPILER_ERROR(compiler,"using src var as dest at line %d", insn->line);
         compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
       }
       if (compiler->vars[var].vartype == ORC_VAR_TYPE_CONST) {
-        ORC_COMPILER_ERROR(compiler,"using const var as dest");
+        ORC_COMPILER_ERROR(compiler,"using const var as dest at line %d", insn->line);
         compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
       }
       if (compiler->vars[var].vartype == ORC_VAR_TYPE_PARAM) {
-        ORC_COMPILER_ERROR(compiler,"using param var as dest");
+        ORC_COMPILER_ERROR(compiler,"using param var as dest at line %d", insn->line);
         compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
       }
       if (opcode->flags & ORC_STATIC_OPCODE_ACCUMULATOR) {
         if (compiler->vars[var].vartype != ORC_VAR_TYPE_ACCUMULATOR) {
-          ORC_COMPILER_ERROR(compiler,"accumulating opcode to non-accumulator dest");
+          ORC_COMPILER_ERROR(compiler,"accumulating opcode to non-accumulator dest at line %d", insn->line);
           compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
         }
       } else {
         if (compiler->vars[var].vartype == ORC_VAR_TYPE_ACCUMULATOR) {
-          ORC_COMPILER_ERROR(compiler,"non-accumulating opcode to accumulator dest");
+          ORC_COMPILER_ERROR(compiler,"non-accumulating opcode to accumulator dest at line %d", insn->line);
           compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
         }
       }
@@ -779,7 +781,7 @@ orc_compiler_rewrite_vars (OrcCompiler *compiler)
       } else {
 #if 0
         if (compiler->vars[var].vartype == ORC_VAR_TYPE_DEST) {
-          ORC_COMPILER_ERROR(compiler,"writing dest more than once");
+          ORC_COMPILER_ERROR(compiler,"writing dest more than once at line %d", insn->line);
           compiler->result = ORC_COMPILE_RESULT_UNKNOWN_PARSE;
         }
 #endif
