@@ -6,6 +6,10 @@
 
 ORC_BEGIN_DECLS
 
+typedef enum {
+  ORC_TARGET_POWERPC_64BIT = (1<<0)
+} OrcTargetPowerPCFlags;
+
 #ifdef ORC_ENABLE_UNSTABLE_API
 
 enum {
@@ -86,6 +90,9 @@ void powerpc_emit_addi (OrcCompiler *compiler, int regd, int rega, int imm);
 void powerpc_emit_lwz (OrcCompiler *compiler, int regd, int rega, int imm);
 void powerpc_emit_stw (OrcCompiler *compiler, int regs, int rega, int offset);
 void powerpc_emit_stwu (OrcCompiler *compiler, int regs, int rega, int offset);
+void powerpc_emit_ld (OrcCompiler *compiler, int regd, int rega, int imm);
+void powerpc_emit_std (OrcCompiler *compiler, int regs, int rega, int offset);
+void powerpc_emit_stdu (OrcCompiler *compiler, int regs, int rega, int offset);
 
 void powerpc_emit_ret (OrcCompiler *compiler);
 void powerpc_emit_b (OrcCompiler *compiler, int label);
@@ -125,8 +132,11 @@ void powerpc_emit_VX_4 (OrcCompiler *p, const char *name, unsigned int insn,
     int d, int a);
 int powerpc_get_constant (OrcCompiler *p, int type, int value);
 int powerpc_get_constant_full (OrcCompiler *p, int value0, int value1, int value2, int value3);
+void powerpc_load_long_constant (OrcCompiler *p, int reg, orc_uint32 a,
+    orc_uint32 b, orc_uint32 c, orc_uint32 d);
 
 /* instructions */
+#define powerpc_emit_vand(p,a,b,c)         powerpc_emit_VX_2 (p, "vand", 0x10000404, a, b, c)
 #define powerpc_emit_vandc(p,a,b,c)        powerpc_emit_VX_2 (p, "vandc", 0x10000444, a, b, c)
 
 #define powerpc_emit_vor(p,a,b,c)          powerpc_emit_VX_2 (p, "vor", 0x10000484, a, b, c)
