@@ -83,7 +83,20 @@ orc_target_c_get_typedefs (void)
     "#else\n"
     "#define ORC_RESTRICT\n"
     "#endif\n"
-    "#endif\n";
+    "#endif\n"
+    "\n"
+    "#ifndef ORC_INTERNAL\n"
+    "#if defined(__SUNPRO_C) && (__SUNPRO_C >= 0x590)\n"
+    "#define ORC_INTERNAL __attribute__((visibility(\"hidden\")))\n"
+    "#elif defined(__SUNPRO_C) && (__SUNPRO_C >= 0x550)\n"
+    "#define ORC_INTERNAL __hidden\n"
+    "#elif defined (__GNUC__)\n"
+    "#define ORC_INTERNAL __attribute__((visibility(\"hidden\")))\n"
+    "#else\n"
+    "#define ORC_INTERNAL\n"
+    "#endif\n"
+    "#endif\n"
+    "\n";
 }
 
 const char *
@@ -230,7 +243,18 @@ orc_compiler_c_assemble (OrcCompiler *compiler)
         break;
       case ORC_VAR_TYPE_TEMP:
         if (!(var->last_use == -1 && var->first_use == 0)) {
-          ORC_ASM_CODE(compiler,"  %s var%d;\n", c_get_type_name(var->size), i);
+          if (var->flags & ORC_VAR_FLAG_VOLATILE_WORKAROUND) {
+            ORC_ASM_CODE(compiler,"#if defined(__APPLE__) && __GNUC__ == 4 && __GNUC_MINOR__ == 2 && defined (__i386__) \n");
+            ORC_ASM_CODE(compiler,"  volatile %s var%d;\n",
+                c_get_type_name(var->size), i);
+            ORC_ASM_CODE(compiler,"#else\n");
+            ORC_ASM_CODE(compiler,"  %s var%d;\n",
+                c_get_type_name(var->size), i);
+            ORC_ASM_CODE(compiler,"#endif\n");
+          } else {
+            ORC_ASM_CODE(compiler,"  %s var%d;\n",
+                c_get_type_name(var->size), i);
+          }
         }
         break;
       case ORC_VAR_TYPE_SRC:
@@ -1011,8 +1035,12 @@ c_rule_splitql (OrcCompiler *p, void *user, OrcInstruction *insn)
   c_get_name_int (dest2, p, insn, insn->dest_args[1]);
   c_get_name_int (src, p, insn, insn->src_args[0]);
 
-  ORC_ASM_CODE(p,"    %s = (%s >> 32) & 0xffffffff;\n", dest1, src);
-  ORC_ASM_CODE(p,"    %s = %s & 0xffffffff;\n", dest2, src);
+  ORC_ASM_CODE(p, "    {\n");
+  ORC_ASM_CODE(p,"       orc_union64 _src;\n");
+  ORC_ASM_CODE(p,"       _src.i = %s;\n", src);
+  ORC_ASM_CODE(p,"       %s = _src.x2[1];\n", dest1);
+  ORC_ASM_CODE(p,"       %s = _src.x2[0];\n", dest2);
+  ORC_ASM_CODE(p, "    }\n");
 }
 
 static void
@@ -1024,8 +1052,12 @@ c_rule_splitlw (OrcCompiler *p, void *user, OrcInstruction *insn)
   c_get_name_int (dest2, p, insn, insn->dest_args[1]);
   c_get_name_int (src, p, insn, insn->src_args[0]);
 
-  ORC_ASM_CODE(p,"    %s = (%s >> 16) & 0xffff;\n", dest1, src);
-  ORC_ASM_CODE(p,"    %s = %s & 0xffff;\n", dest2, src);
+  ORC_ASM_CODE(p, "    {\n");
+  ORC_ASM_CODE(p,"       orc_union32 _src;\n");
+  ORC_ASM_CODE(p,"       _src.i = %s;\n", src);
+  ORC_ASM_CODE(p,"       %s = _src.x2[1];\n", dest1);
+  ORC_ASM_CODE(p,"       %s = _src.x2[0];\n", dest2);
+  ORC_ASM_CODE(p, "    }\n");
 }
 
 static void
@@ -1037,8 +1069,102 @@ c_rule_splitwb (OrcCompiler *p, void *user, OrcInstruction *insn)
   c_get_name_int (dest2, p, insn, insn->dest_args[1]);
   c_get_name_int (src, p, insn, insn->src_args[0]);
 
-  ORC_ASM_CODE(p,"    %s = (%s >> 8) & 0xff;\n", dest1, src);
-  ORC_ASM_CODE(p,"    %s = %s & 0xff;\n", dest2, src);
+  ORC_ASM_CODE(p, "    {\n");
+  ORC_ASM_CODE(p,"       orc_union16 _src;\n");
+  ORC_ASM_CODE(p,"       _src.i = %s;\n", src);
+  ORC_ASM_CODE(p,"       %s = _src.x2[1];\n", dest1);
+  ORC_ASM_CODE(p,"       %s = _src.x2[0];\n", dest2);
+  ORC_ASM_CODE(p, "    }\n");
+}
+
+static void
+c_rule_select0ql (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  char dest[40], src[40];
+
+  c_get_name_int (dest, p, insn, insn->dest_args[0]);
+  c_get_name_int (src, p, insn, insn->src_args[0]);
+
+  ORC_ASM_CODE(p, "    {\n");
+  ORC_ASM_CODE(p,"       orc_union64 _src;\n");
+  ORC_ASM_CODE(p,"       _src.i = %s;\n", src);
+  ORC_ASM_CODE(p,"       %s = _src.x2[0];\n", dest);
+  ORC_ASM_CODE(p, "    }\n");
+}
+
+static void
+c_rule_select1ql (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  char dest[40], src[40];
+
+  c_get_name_int (dest, p, insn, insn->dest_args[0]);
+  c_get_name_int (src, p, insn, insn->src_args[0]);
+
+  ORC_ASM_CODE(p, "    {\n");
+  ORC_ASM_CODE(p,"       orc_union64 _src;\n");
+  ORC_ASM_CODE(p,"       _src.i = %s;\n", src);
+  ORC_ASM_CODE(p,"       %s = _src.x2[1];\n", dest);
+  ORC_ASM_CODE(p, "    }\n");
+}
+
+static void
+c_rule_select0lw (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  char dest[40], src[40];
+
+  c_get_name_int (dest, p, insn, insn->dest_args[0]);
+  c_get_name_int (src, p, insn, insn->src_args[0]);
+
+  ORC_ASM_CODE(p, "    {\n");
+  ORC_ASM_CODE(p,"       orc_union32 _src;\n");
+  ORC_ASM_CODE(p,"       _src.i = %s;\n", src);
+  ORC_ASM_CODE(p,"       %s = _src.x2[0];\n", dest);
+  ORC_ASM_CODE(p, "    }\n");
+}
+
+static void
+c_rule_select1lw (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  char dest[40], src[40];
+
+  c_get_name_int (dest, p, insn, insn->dest_args[0]);
+  c_get_name_int (src, p, insn, insn->src_args[0]);
+
+  ORC_ASM_CODE(p, "    {\n");
+  ORC_ASM_CODE(p,"       orc_union32 _src;\n");
+  ORC_ASM_CODE(p,"       _src.i = %s;\n", src);
+  ORC_ASM_CODE(p,"       %s = _src.x2[1];\n", dest);
+  ORC_ASM_CODE(p, "    }\n");
+}
+
+static void
+c_rule_select0wb (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  char dest[40], src[40];
+
+  c_get_name_int (dest, p, insn, insn->dest_args[0]);
+  c_get_name_int (src, p, insn, insn->src_args[0]);
+
+  ORC_ASM_CODE(p, "    {\n");
+  ORC_ASM_CODE(p,"       orc_union16 _src;\n");
+  ORC_ASM_CODE(p,"       _src.i = %s;\n", src);
+  ORC_ASM_CODE(p,"       %s = _src.x2[0];\n", dest);
+  ORC_ASM_CODE(p, "    }\n");
+}
+
+static void
+c_rule_select1wb (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  char dest[40], src[40];
+
+  c_get_name_int (dest, p, insn, insn->dest_args[0]);
+  c_get_name_int (src, p, insn, insn->src_args[0]);
+
+  ORC_ASM_CODE(p, "    {\n");
+  ORC_ASM_CODE(p,"       orc_union16 _src;\n");
+  ORC_ASM_CODE(p,"       _src.i = %s;\n", src);
+  ORC_ASM_CODE(p,"       %s = _src.x2[1];\n", dest);
+  ORC_ASM_CODE(p, "    }\n");
 }
 
 static void
@@ -1116,7 +1242,7 @@ c_rule_convlf (OrcCompiler *p, void *user, OrcInstruction *insn)
   c_get_name_float (dest, p, insn, insn->dest_args[0]);
   c_get_name_int (src1, p, insn, insn->src_args[0]);
 
-  ORC_ASM_CODE(p,"     %s = %s;\n", dest, src1);
+  ORC_ASM_CODE(p,"    %s = %s;\n", dest, src1);
 }
 
 static void
@@ -1127,7 +1253,7 @@ c_rule_convld (OrcCompiler *p, void *user, OrcInstruction *insn)
   c_get_name_float (dest, p, insn, insn->dest_args[0]);
   c_get_name_int (src1, p, insn, insn->src_args[0]);
 
-  ORC_ASM_CODE(p,"     %s = %s;\n", dest, src1);
+  ORC_ASM_CODE(p,"    %s = %s;\n", dest, src1);
 }
 
 static void
@@ -1300,6 +1426,58 @@ c_rule_swaplq (OrcCompiler *p, void *user, OrcInstruction *insn)
       dest, src, src);
 }
 
+static void
+c_rule_mergelq (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  char dest[40], src1[40], src2[40];
+
+  c_get_name_int (dest, p, insn, insn->dest_args[0]);
+  c_get_name_int (src1, p, insn, insn->src_args[0]);
+  c_get_name_int (src2, p, insn, insn->src_args[1]);
+
+  ORC_ASM_CODE(p, "    {\n");
+  ORC_ASM_CODE(p,"       orc_union64 _dest;\n");
+  ORC_ASM_CODE(p,"       _dest.x2[0] = %s;\n", src1);
+  ORC_ASM_CODE(p,"       _dest.x2[1] = %s;\n", src2);
+  ORC_ASM_CODE(p,"       %s = _dest.i;\n", dest);
+  ORC_ASM_CODE(p, "    }\n");
+}
+
+static void
+c_rule_mergewl (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  char dest[40], src1[40], src2[40];
+
+  c_get_name_int (dest, p, insn, insn->dest_args[0]);
+  c_get_name_int (src1, p, insn, insn->src_args[0]);
+  c_get_name_int (src2, p, insn, insn->src_args[1]);
+
+  ORC_ASM_CODE(p, "    {\n");
+  ORC_ASM_CODE(p,"       orc_union32 _dest;\n");
+  ORC_ASM_CODE(p,"       _dest.x2[0] = %s;\n", src1);
+  ORC_ASM_CODE(p,"       _dest.x2[1] = %s;\n", src2);
+  ORC_ASM_CODE(p,"       %s = _dest.i;\n", dest);
+  ORC_ASM_CODE(p, "    }\n");
+}
+
+static void
+c_rule_mergebw (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  char dest[40], src1[40], src2[40];
+
+  c_get_name_int (dest, p, insn, insn->dest_args[0]);
+  c_get_name_int (src1, p, insn, insn->src_args[0]);
+  c_get_name_int (src2, p, insn, insn->src_args[1]);
+
+  ORC_ASM_CODE(p, "    {\n");
+  ORC_ASM_CODE(p,"       orc_union16 _dest;\n");
+  ORC_ASM_CODE(p,"       _dest.x2[0] = %s;\n", src1);
+  ORC_ASM_CODE(p,"       _dest.x2[1] = %s;\n", src2);
+  ORC_ASM_CODE(p,"       %s = _dest.i;\n", dest);
+  ORC_ASM_CODE(p, "    }\n");
+}
+
+
 static OrcTarget c_target = {
   "c",
   FALSE,
@@ -1394,6 +1572,12 @@ orc_c_init (void)
   orc_rule_register (rule_set, "splitql", c_rule_splitql, NULL);
   orc_rule_register (rule_set, "splitlw", c_rule_splitlw, NULL);
   orc_rule_register (rule_set, "splitwb", c_rule_splitwb, NULL);
+  orc_rule_register (rule_set, "select0ql", c_rule_select0ql, NULL);
+  orc_rule_register (rule_set, "select1ql", c_rule_select1ql, NULL);
+  orc_rule_register (rule_set, "select0lw", c_rule_select0lw, NULL);
+  orc_rule_register (rule_set, "select1lw", c_rule_select1lw, NULL);
+  orc_rule_register (rule_set, "select0wb", c_rule_select0wb, NULL);
+  orc_rule_register (rule_set, "select1wb", c_rule_select1wb, NULL);
   orc_rule_register (rule_set, "splatbw", c_rule_splatbw, NULL);
   orc_rule_register (rule_set, "splatbl", c_rule_splatbl, NULL);
   orc_rule_register (rule_set, "splatw3q", c_rule_splatw3q, NULL);
@@ -1411,5 +1595,8 @@ orc_c_init (void)
   orc_rule_register (rule_set, "maxd", c_rule_maxd, NULL);
   orc_rule_register (rule_set, "swapwl", c_rule_swapwl, NULL);
   orc_rule_register (rule_set, "swaplq", c_rule_swaplq, NULL);
+  orc_rule_register (rule_set, "mergebw", c_rule_mergebw, NULL);
+  orc_rule_register (rule_set, "mergewl", c_rule_mergewl, NULL);
+  orc_rule_register (rule_set, "mergelq", c_rule_mergelq, NULL);
 }
 
