@@ -227,23 +227,6 @@ orc_arm_storel (OrcCompiler *compiler, int dest, int offset, int src1)
   orc_arm_emit (compiler, code);
 }
 
-void
-orc_arm_loadw (OrcCompiler *compiler, int dest, int src1, int offset)
-{
-  orc_uint32 code;
-
-  code = 0xe1d000b0;
-  code |= (src1&0xf) << 16;
-  code |= (dest&0xf) << 12;
-  code |= (offset&0xf0) << 4;
-  code |= offset&0x0f;
-
-  ORC_ASM_CODE(compiler,"  ldrh %s, [%s, #%d]\n",
-      orc_arm_reg_name (dest),
-      orc_arm_reg_name (src1), offset);
-  orc_arm_emit (compiler, code);
-}
-
 void
 orc_arm_storew (OrcCompiler *compiler, int dest, int offset, int src1)
 {
