@@ -32,7 +32,7 @@ sse_rule_loadpX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
           (int)ORC_STRUCT_OFFSET(OrcExecutor, params[insn->src_args[0]]),
           compiler->exec_reg, reg, FALSE);
 #ifndef MMX
-      orc_x86_emit_movhps_memoffset_sse (compiler,
+      orc_sse_emit_movhps_load_memoffset (compiler,
           (int)ORC_STRUCT_OFFSET(OrcExecutor,
             params[insn->src_args[0] + (ORC_VAR_T1 - ORC_VAR_P1)]),
           compiler->exec_reg, reg);
@@ -106,7 +106,7 @@ sse_rule_loadX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
     case 1:
       orc_x86_emit_mov_memoffset_reg (compiler, 1, offset, ptr_reg,
           compiler->gp_tmpreg);
-      orc_x86_emit_mov_reg_sse (compiler, compiler->gp_tmpreg, dest->alloc);
+      orc_sse_emit_movd_load_register (compiler, compiler->gp_tmpreg, dest->alloc);
       break;
     case 2:
       orc_sse_emit_pxor (compiler, dest->alloc, dest->alloc);
@@ -125,7 +125,7 @@ sse_rule_loadX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
           dest->alloc, src->is_aligned);
       break;
     default:
-      ORC_COMPILER_ERROR(compiler,"bad load size %d",
+      orc_compiler_error (compiler, "bad load size %d",
           src->size << compiler->loop_shift);
       break;
   }
@@ -142,7 +142,8 @@ sse_rule_loadoffX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
   int offset = 0;
 
   if (compiler->vars[insn->src_args[1]].vartype != ORC_VAR_TYPE_CONST) {
-    ORC_COMPILER_ERROR(compiler, "Rule only works with consts");
+    orc_compiler_error (compiler, "code generation rule for %s only works with constant offset",
+        insn->opcode->name);
     return;
   }
 
@@ -161,7 +162,7 @@ sse_rule_loadoffX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
     case 1:
       orc_x86_emit_mov_memoffset_reg (compiler, 1, offset, ptr_reg,
           compiler->gp_tmpreg);
-      orc_x86_emit_mov_reg_sse (compiler, compiler->gp_tmpreg, dest->alloc);
+      orc_sse_emit_movd_load_register (compiler, compiler->gp_tmpreg, dest->alloc);
       break;
     case 2:
       orc_sse_emit_pxor (compiler, dest->alloc, dest->alloc);
@@ -180,7 +181,7 @@ sse_rule_loadoffX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
           dest->alloc, src->is_aligned);
       break;
     default:
-      ORC_COMPILER_ERROR(compiler,"bad load size %d",
+      orc_compiler_error (compiler,"bad load size %d",
           src->size << compiler->loop_shift);
       break;
   }
@@ -237,7 +238,7 @@ sse_rule_loadupib (OrcCompiler *compiler, void *user, OrcInstruction *insn)
           tmp, FALSE);
       break;
     default:
-      ORC_COMPILER_ERROR(compiler,"bad load size %d",
+      orc_compiler_error(compiler,"bad load size %d",
           src->size << compiler->loop_shift);
       break;
   }
@@ -271,7 +272,7 @@ sse_rule_loadupdb (OrcCompiler *compiler, void *user, OrcInstruction *insn)
     case 2:
       orc_x86_emit_mov_memoffset_reg (compiler, 1, offset, ptr_reg,
           compiler->gp_tmpreg);
-      orc_x86_emit_mov_reg_sse (compiler, compiler->gp_tmpreg, dest->alloc);
+      orc_sse_emit_movd_load_register (compiler, compiler->gp_tmpreg, dest->alloc);
       break;
     case 4:
       orc_sse_emit_pinsrw_memoffset (compiler, 0, offset, ptr_reg, dest->alloc);
@@ -289,7 +290,7 @@ sse_rule_loadupdb (OrcCompiler *compiler, void *user, OrcInstruction *insn)
           dest->alloc, src->is_aligned);
       break;
     default:
-      ORC_COMPILER_ERROR(compiler,"bad load size %d",
+      orc_compiler_error(compiler,"bad load size %d",
           src->size << compiler->loop_shift);
       break;
   }
@@ -328,9 +329,10 @@ sse_rule_storeX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
     case 1:
       /* FIXME we might be using ecx twice here */
       if (ptr_reg == compiler->gp_tmpreg) {
-        ORC_COMPILER_ERROR(compiler,"unimplemented");
+        orc_compiler_error (compiler, "unimplemented corner case in %s",
+            insn->opcode->name);
       }
-      orc_x86_emit_mov_sse_reg (compiler, src->alloc, compiler->gp_tmpreg);
+      orc_sse_emit_movd_store_register (compiler, src->alloc, compiler->gp_tmpreg);
       orc_x86_emit_mov_reg_memoffset (compiler, 1, compiler->gp_tmpreg,
           offset, ptr_reg);
       break;
@@ -341,9 +343,10 @@ sse_rule_storeX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
       } else {
         /* FIXME we might be using ecx twice here */
         if (ptr_reg == compiler->gp_tmpreg) {
-          ORC_COMPILER_ERROR(compiler,"unimplemented");
+          orc_compiler_error(compiler, "unimplemented corner case in %s",
+              insn->opcode->name);
         } 
-        orc_x86_emit_mov_sse_reg (compiler, src->alloc, compiler->gp_tmpreg);
+        orc_sse_emit_movd_store_register (compiler, src->alloc, compiler->gp_tmpreg);
         orc_x86_emit_mov_reg_memoffset (compiler, 2, compiler->gp_tmpreg,
             offset, ptr_reg);
       }
@@ -361,7 +364,7 @@ sse_rule_storeX (OrcCompiler *compiler, void *user, OrcInstruction *insn)
           dest->is_aligned, dest->is_uncached);
       break;
     default:
-      ORC_COMPILER_ERROR(compiler,"bad size");
+      orc_compiler_error (compiler, "bad size");
       break;
   }
 
@@ -378,7 +381,7 @@ sse_rule_ldresnearl (OrcCompiler *compiler, void *user, OrcInstruction *insn)
   int tmp2 = orc_compiler_get_temp_reg (compiler);
   int tmpc;
 
-  orc_x86_emit_mov_sse_reg (compiler, X86_XMM6, compiler->gp_tmpreg);
+  orc_sse_emit_movd_store_register (compiler, X86_XMM6, compiler->gp_tmpreg);
   orc_x86_emit_sar_imm_reg (compiler, 4, 16, compiler->gp_tmpreg);
 
   orc_sse_emit_movdqu_load_memindex (compiler, 0, src->ptr_register,
@@ -490,7 +493,7 @@ sse_rule_ldreslinl (OrcCompiler *compiler, void *user, OrcInstruction *insn)
     orc_sse_emit_pshufd (compiler, ORC_SSE_SHUF(3,2,3,2), tmp, tmp2);
     orc_sse_emit_psubw (compiler, tmp, tmp2);
 
-    orc_x86_emit_mov_reg_sse (compiler, src->ptr_offset, tmp);
+    orc_sse_emit_movd_load_register (compiler, src->ptr_offset, tmp);
     orc_sse_emit_pshuflw (compiler, ORC_SSE_SHUF(0,0,0,0), tmp, tmp);
     orc_sse_emit_psrlw_imm (compiler, 8, tmp);
     orc_sse_emit_pmullw (compiler, tmp2, tmp);
@@ -525,7 +528,7 @@ sse_rule_ldreslinl (OrcCompiler *compiler, void *user, OrcInstruction *insn)
     for(i=0;i<(1<<compiler->loop_shift);i+=2){
       orc_x86_emit_mov_memoffset_sse (compiler, 8, 0,
           src->ptr_register, tmp, FALSE);
-      orc_x86_emit_mov_reg_sse (compiler, src->ptr_offset, tmp4);
+      orc_sse_emit_movd_load_register (compiler, src->ptr_offset, tmp4);
 
       if (compiler->vars[increment_var].vartype == ORC_VAR_TYPE_PARAM) {
         orc_x86_emit_add_memoffset_reg (compiler, 4,
@@ -621,7 +624,7 @@ mmx_rule_ldreslinl (OrcCompiler *compiler, void *user, OrcInstruction *insn)
     orc_mmx_emit_punpcklbw (compiler, zero, tmp2);
     orc_mmx_emit_psubw (compiler, tmp, tmp2);
 
-    orc_x86_emit_mov_reg_mmx (compiler, src->ptr_offset, tmp);
+    orc_sse_emit_movd_load_register (compiler, src->ptr_offset, tmp);
     orc_mmx_emit_pshufw (compiler, ORC_MMX_SHUF(0,0,0,0), tmp, tmp);
     orc_mmx_emit_psrlw_imm (compiler, 8, tmp);
     orc_mmx_emit_pmullw (compiler, tmp2, tmp);
@@ -837,6 +840,7 @@ sse_rule_accsadubl (OrcCompiler *p, void *user, OrcInstruction *insn)
   orc_sse_emit_paddd (p, tmp, dest);
 }
 
+#ifndef MMX
 static void
 sse_rule_signX_ssse3 (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
@@ -848,14 +852,15 @@ sse_rule_signX_ssse3 (OrcCompiler *p, void *user, OrcInstruction *insn)
 
   tmpc = orc_compiler_get_temp_constant (p, 1<<type, 1);
   if (src == dest) {
-    orc_x86_emit_cpuinsn (p, opcodes[type], 0, src, tmpc);
+    orc_x86_emit_cpuinsn_size (p, opcodes[type], 16, src, tmpc);
     orc_sse_emit_movdqa (p, tmpc, dest);
   } else {
     /* FIXME this would be a good opportunity to not chain src to dest */
     orc_sse_emit_movdqa (p, tmpc, dest);
-    orc_x86_emit_cpuinsn (p, opcodes[type], 0, src, dest);
+    orc_x86_emit_cpuinsn_size (p, opcodes[type], 16, src, dest);
   }
 }
+#endif
 
 static void
 sse_rule_signw_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
@@ -939,8 +944,8 @@ sse_rule_shift (OrcCompiler *p, void *user, OrcInstruction *insn)
     ORC_X86_psrad_imm, ORC_X86_psllq_imm, ORC_X86_psrlq_imm };
 
   if (p->vars[insn->src_args[1]].vartype == ORC_VAR_TYPE_CONST) {
-    orc_x86_emit_cpuinsn (p, opcodes_imm[type],
-        p->vars[insn->src_args[1]].value.i, 0,
+    orc_x86_emit_cpuinsn_imm (p, opcodes_imm[type],
+        p->vars[insn->src_args[1]].value.i, 16,
         p->vars[insn->dest_args[0]].alloc);
   } else if (p->vars[insn->src_args[1]].vartype == ORC_VAR_TYPE_PARAM) {
     int tmp = orc_compiler_get_temp_reg (p);
@@ -951,10 +956,11 @@ sse_rule_shift (OrcCompiler *p, void *user, OrcInstruction *insn)
         (int)ORC_STRUCT_OFFSET(OrcExecutor, params[insn->src_args[1]]),
         p->exec_reg, tmp, FALSE);
 
-    orc_x86_emit_cpuinsn (p, opcodes[type], 0, tmp,
+    orc_x86_emit_cpuinsn_size (p, opcodes[type], 16, tmp,
         p->vars[insn->dest_args[0]].alloc);
   } else {
-    ORC_COMPILER_ERROR(p,"rule only works with constants or params");
+    orc_compiler_error (p, "code generation rule for %s only works with "
+        "constant or parameter shifts", insn->opcode->name);
     p->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
   }
 }
@@ -971,7 +977,8 @@ sse_rule_shlb (OrcCompiler *p, void *user, OrcInstruction *insn)
         0xff&(0xff<<p->vars[insn->src_args[1]].value.i));
     orc_sse_emit_pand (p, tmp, dest);
   } else {
-    ORC_COMPILER_ERROR(p,"rule only works with constants");
+    orc_compiler_error (p, "code generation rule for %s only works with "
+        "constant shifts", insn->opcode->name);
     p->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
   }
 }
@@ -994,7 +1001,8 @@ sse_rule_shrsb (OrcCompiler *p, void *user, OrcInstruction *insn)
 
     orc_sse_emit_por (p, tmp, dest);
   } else {
-    ORC_COMPILER_ERROR(p,"rule only works with constants");
+    orc_compiler_error (p, "code generation rule for %s only works with "
+        "constant shifts", insn->opcode->name);
     p->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
   }
 }
@@ -1011,7 +1019,8 @@ sse_rule_shrub (OrcCompiler *p, void *user, OrcInstruction *insn)
         (0xff>>p->vars[insn->src_args[1]].value.i));
     orc_sse_emit_pand (p, tmp, dest);
   } else {
-    ORC_COMPILER_ERROR(p,"rule only works with constants");
+    orc_compiler_error (p, "code generation rule for %s only works with "
+        "constant shifts", insn->opcode->name);
     p->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
   }
 }
@@ -1035,7 +1044,8 @@ sse_rule_shrsq (OrcCompiler *p, void *user, OrcInstruction *insn)
     orc_sse_emit_psrlq_imm (p, p->vars[insn->src_args[1]].value.i, dest);
     orc_sse_emit_por (p, tmp, dest);
   } else {
-    ORC_COMPILER_ERROR(p,"rule only works with constants");
+    orc_compiler_error (p, "code generation rule for %s only works with "
+        "constant shifts", insn->opcode->name);
     p->result = ORC_COMPILE_RESULT_UNKNOWN_COMPILE;
   }
 }
@@ -1529,6 +1539,7 @@ sse_rule_mulhsl (OrcCompiler *p, void *user, OrcInstruction *insn)
 }
 #endif
 
+#ifndef MMX
 static void
 sse_rule_mulhsl_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
 {
@@ -1547,16 +1558,17 @@ sse_rule_mulhsl_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
 
   for(i=0;i<(1<<p->insn_shift);i++) {
     orc_x86_emit_mov_memoffset_reg (p, 4, offset + 4*i, p->exec_reg, X86_EAX);
-    orc_x86_emit_cpuinsn_load_memoffset (p, ORC_X86_imul_rm, 4, 0,
-        offset + 16 + 4*i, p->exec_reg, -1);
+    orc_x86_emit_cpuinsn_memoffset (p, ORC_X86_imul_rm, 4,
+        offset + 16 + 4*i, p->exec_reg);
     orc_x86_emit_mov_reg_memoffset (p, 4, X86_EDX, offset + 4*i, p->exec_reg);
   }
 
   orc_x86_emit_mov_memoffset_sse (p, 16, offset, p->exec_reg,
       p->vars[insn->dest_args[0]].alloc, FALSE);
-  orc_x86_emit_mov_memoffset_reg (p, 8, offset + 32, p->exec_reg, X86_EAX);
-  orc_x86_emit_mov_memoffset_reg (p, 8, offset + 40, p->exec_reg, X86_EDX);
+  orc_x86_emit_mov_memoffset_reg (p, regsize, offset + 32, p->exec_reg, X86_EAX);
+  orc_x86_emit_mov_memoffset_reg (p, regsize, offset + 40, p->exec_reg, X86_EDX);
 }
+#endif
 
 #ifndef MMX
 static void
@@ -1590,6 +1602,38 @@ sse_rule_mulslq (OrcCompiler *p, void *user, OrcInstruction *insn)
   orc_sse_emit_pmuldq (p, tmp, dest);
 }
 
+#ifndef MMX
+static void
+sse_rule_mulslq_slow (OrcCompiler *p, void *user, OrcInstruction *insn)
+{
+  int i;
+  int regsize = p->is_64bit ? 8 : 4;
+  int offset = ORC_STRUCT_OFFSET(OrcExecutor,arrays[ORC_VAR_T1]);
+
+  orc_x86_emit_mov_sse_memoffset (p, 8, p->vars[insn->src_args[0]].alloc,
+      offset, p->exec_reg, FALSE, FALSE);
+  orc_x86_emit_mov_sse_memoffset (p, 8, p->vars[insn->src_args[1]].alloc,
+      offset + 8, p->exec_reg, FALSE, FALSE);
+  orc_x86_emit_mov_reg_memoffset (p, regsize, X86_EAX, offset + 32,
+      p->exec_reg);
+  orc_x86_emit_mov_reg_memoffset (p, regsize, X86_EDX, offset + 40,
+      p->exec_reg);
+
+  for(i=0;i<(1<<p->insn_shift);i++) {
+    orc_x86_emit_mov_memoffset_reg (p, 4, offset + 4*i, p->exec_reg, X86_EAX);
+    orc_x86_emit_cpuinsn_memoffset (p, ORC_X86_imul_rm, 4,
+        offset + 8 + 4*i, p->exec_reg);
+    orc_x86_emit_mov_reg_memoffset (p, 4, X86_EAX, offset + 16 + 8*i, p->exec_reg);
+    orc_x86_emit_mov_reg_memoffset (p, 4, X86_EDX, offset + 16 + 8*i + 4, p->exec_reg);
+  }
+
+  orc_x86_emit_mov_memoffset_sse (p, 16, offset + 16, p->exec_reg,
+      p->vars[insn->dest_args[0]].alloc, FALSE);
+  orc_x86_emit_mov_memoffset_reg (p, regsize, offset + 32, p->exec_reg, X86_EAX);
+  orc_x86_emit_mov_memoffset_reg (p, regsize, offset + 40, p->exec_reg, X86_EDX);
+}
+#endif
+
 #ifndef MMX
 static void
 sse_rule_mululq (OrcCompiler *p, void *user, OrcInstruction *insn)
@@ -2850,9 +2894,10 @@ orc_compiler_sse_register_rules (OrcTarget *target)
   orc_rule_register (rule_set, "shrsb", sse_rule_shrsb, NULL);
   orc_rule_register (rule_set, "shrub", sse_rule_shrub, NULL);
   orc_rule_register (rule_set, "mulll", sse_rule_mulll_slow, NULL);
-  orc_rule_register (rule_set, "mulhsl", sse_rule_mulhsl_slow, NULL);
 #ifndef MMX
+  orc_rule_register (rule_set, "mulhsl", sse_rule_mulhsl_slow, NULL);
   orc_rule_register (rule_set, "mulhul", sse_rule_mulhul, NULL);
+  orc_rule_register (rule_set, "mulslq", sse_rule_mulslq_slow, NULL);
 #endif
   orc_rule_register (rule_set, "mullb", sse_rule_mullb, NULL);
   orc_rule_register (rule_set, "mulhsb", sse_rule_mulhsb, NULL);
@@ -2875,9 +2920,11 @@ orc_compiler_sse_register_rules (OrcTarget *target)
   rule_set = orc_rule_set_new (orc_opcode_set_get("sys"), target,
       ORC_TARGET_SSE_SSSE3);
 
+#ifndef MMX
   orc_rule_register (rule_set, "signb", sse_rule_signX_ssse3, (void *)0);
   orc_rule_register (rule_set, "signw", sse_rule_signX_ssse3, (void *)1);
   orc_rule_register (rule_set, "signl", sse_rule_signX_ssse3, (void *)2);
+#endif
   REG(absb);
   REG(absw);
   REG(absl);
